@@ -44,7 +44,7 @@
  */
 
 
-#include <json-c/json.h>
+#include <json.h>
 #include <arpa/inet.h>
 #include <netdb.h>
 #include "network.h"
