@@ -44,7 +44,7 @@
  */
 
 
-#include <json-c/json.h>
+#include <json.h>
 #include "network.h"
 #include "ssl_h.h"
 #include "sslprint.h"
